@@ -56,6 +56,7 @@ import tty
 import signal
 import re
 import time
+import base64
 import collections
 
 PY2 = sys.version_info < (3, 0)
@@ -84,7 +85,7 @@ except (ImportError, ValueError):
     import utils
 
 
-PROTO_VERSION = '0.03'
+PROTO_VERSION = '0.1'
 CLIENT = 'flootty'
 INITIAL_RECONNECT_DELAY = 1000
 FD_READ_BYTES = 65536
@@ -154,11 +155,11 @@ def die(*args):
     sys.exit(1)
 
 
-def parse_url(room_url):
+def parse_url(workspace_url):
     secure = True
     owner = None
-    room_name = None
-    parsed_url = urlparse(room_url)
+    workspace_name = None
+    parsed_url = urlparse(workspace_url)
     port = parsed_url.port
     if not port:
         port = 3448
@@ -166,16 +167,20 @@ def parse_url(room_url):
         if not port:
             port = 3148
         secure = False
-    result = re.match('^/r/([-\w]+)/([-\w]+)/?$', parsed_url.path)
+    result = re.match('^/([-\@\+\.\w]+)/([-\w]+)/?$', parsed_url.path)
+    if not result:
+        result = re.match('^/r/([-\@\+\.\w]+)/([-\w]+)/?$', parsed_url.path)
+
     if result:
-        (owner, room_name) = result.groups()
+        (owner, workspace_name) = result.groups()
     else:
-        raise ValueError('%s is not a valid Floobits URL' % room_url)
+        raise ValueError('%s is not a valid Floobits URL' % workspace_url)
+
     return {
         'host': parsed_url.hostname,
         'owner': owner,
         'port': port,
-        'room': room_name,
+        'workspace': workspace_name,
         'secure': secure,
     }
 
@@ -212,7 +217,7 @@ def main():
                       help="The terminal name to create")
 
     parser.add_option("-w", "--workspace",
-                      dest="room",
+                      dest="workspace",
                       help="The workspace name")
 
     parser.add_option("-o", "--owner",
@@ -238,7 +243,7 @@ def main():
                       help="Do not use this option unless you know what you are doing!")
 
     parser.add_option("--url",
-                      dest="room_url",
+                      dest="workspace_url",
                       default=None,
                       help="The URL of the workspace to connect to. This is a convenience for copy-pasting from the browser.")
 
@@ -253,13 +258,13 @@ def main():
 
     term_name = args and args[0] or default_term_name
 
-    if options.room and options.owner and options.room_url:
+    if options.workspace and options.owner and options.workspace_url:
         parser.error("You can either specify --workspace and --owner, or --url, but not both.")
 
-    if not options.room or not options.owner:
+    if not options.workspace or not options.owner:
         floo = {}
-        if options.room_url:
-            floo = parse_url(options.room_url)
+        if options.workspace_url:
+            floo = parse_url(options.workspace_url)
         else:
             for floo_path in walk_up(os.path.realpath('.')):
                 try:
@@ -269,29 +274,26 @@ def main():
                     pass
                 else:
                     break
-        options.room = floo.get('room')
+        options.workspace = floo.get('workspace')
         options.owner = floo.get('owner')
         if not options.port:
             options.port = floo.get('port')
         if not options.host:
             options.host = floo.get('host')
 
-    if not options.room or not options.owner:
-        try:
-            now_editing = api.get_now_editing_workspaces()
-            now_editing = json.loads(now_editing.read().decode('utf-8'))
-            if len(now_editing) == 1:
-                options.room = now_editing[0]['name']
-                options.owner = now_editing[0]['owner']
-            # TODO: list possible workspaces to join if > 1 is active
-        except Exception as e:
-            raise e
+    if not options.workspace or not options.owner:
+        now_editing = api.get_now_editing_workspaces()
+        now_editing = json.loads(now_editing.read().decode('utf-8'))
+        if len(now_editing) == 1:
+            options.workspace = now_editing[0]['name']
+            options.owner = now_editing[0]['owner']
+        # TODO: list possible workspaces to join if > 1 is active
 
     if options.list:
         if len(term_name) != 0:
             die("I don't understand why you gave me a positional argument.")
 
-    for opt in ['room', 'owner', 'username', 'secret']:
+    for opt in ['workspace', 'owner', 'username', 'secret']:
         if not getattr(options, opt):
             parser.error('%s not given' % opt)
 
@@ -344,7 +346,7 @@ class Flootty(object):
 
         self.host = options.host
         self.port = int(options.port)
-        self.room = options.room
+        self.workspace = options.workspace
         self.owner = options.owner
         self.options = options
         self.term_name = term_name
@@ -480,13 +482,21 @@ class Flootty(object):
             return out('no name in data?!?')
         func = getattr(self, "on_%s" % (name), None)
         if not func:
-            #out('unknown name %s data: %s' % (name, data))
             return
         func(data)
 
     def on_room_info(self, ri):
         self.authed = True
         self.ri = ri
+
+        def list_terms(terms):
+            term_name = ""
+            for term_id, term in terms.items():
+                owner = str(term['owner'])
+                term_name = term['term_name']
+                out('terminal %s created by %s' % (term['term_name'], ri['users'][owner]['username']))
+            return term_name
+
         if self.options.create:
             buf = self._get_pty_size()
             term_name = self.term_name
@@ -498,10 +508,8 @@ class Flootty(object):
             self.term_name = term_name
             return self.transport('create_term', {'term_name': self.term_name, 'size': [buf[1], buf[0]]})
         elif self.options.list:
-            out('Terminals in %s::%s' % (self.owner, self.room))
-            for term_id, term in ri['terms'].items():
-                owner = str(term['owner'])
-                out('terminal %s created by %s' % (term['term_name'], ri['users'][owner]['username']))
+            out('Terminals in %s::%s' % (self.owner, self.workspace))
+            list_terms(ri['terms'])
             return die()
         elif not self.term_name:
             if len(ri['terms']) == 0:
@@ -520,10 +528,8 @@ class Flootty(object):
                 self.term_name = term['term_name']
             else:
                 out('More than one active term exists in this workspace.')
-                for term_id, term in ri['terms'].items():
-                    owner = str(term['owner'])
-                    out('terminal %s created by %s' % (term['term_name'], ri['users'][owner]))
-                    die('Please pick a workspace like so: flootty [super_awesome_name]')
+                example_name = list_terms(ri['terms'])
+                die('Please pick a workspace like so: flootty %s' % example_name)
         else:
             for term_id, term in ri['terms'].items():
                 if term['term_name'] == self.term_name:
@@ -564,7 +570,7 @@ class Flootty(object):
             return
         if not self.options.create:
             return
-        self.handle_stdio(data['data'])
+        self.handle_stdio(base64.b64decode(data['data']))
 
     def on_term_stdout(self, data):
         if data.get('id') != self.term_id:
@@ -613,7 +619,7 @@ class Flootty(object):
             'name': 'auth',
             'username': self.options.username,
             'secret': self.options.secret,
-            'room': self.room,
+            'room': self.workspace,
             'room_owner': self.owner,
             'client': CLIENT,
             'platform': sys.platform,
@@ -645,7 +651,7 @@ class Flootty(object):
         self.add_fd(self.sock, reader=self.cloud_read, writer=self.cloud_write, errer=self.cloud_err, name='net')
         self.reconnect_delay = INITIAL_RECONNECT_DELAY
 
-    def room_url(self):
+    def workspace_url(self):
         proto = {True: "https", False: "http"}
         proto_str = proto[self.options.use_ssl]
         port_str = ''
@@ -655,10 +661,10 @@ class Flootty(object):
         else:
             if self.port != 3148:
                 port_str = ':%s' % self.port
-        return '%s://%s%s/r/%s/%s/' % (proto_str, self.host, port_str, self.owner, self.room)
+        return '%s://%s%s/%s/%s/' % (proto_str, self.host, port_str, self.owner, self.workspace)
 
     def join_term(self):
-        out('Successfully joined %s' % (self.room_url()))
+        out('Successfully joined %s' % (self.workspace_url()))
         self.orig_stdout_atts = tty.tcgetattr(sys.stdout)
         stdout = sys.stdout.fileno()
         tty.setraw(stdout)
@@ -674,9 +680,7 @@ class Flootty(object):
         def ship_stdin(fd):
             data = read(fd)
             if data:
-                if not PY2:
-                    data = data.decode('utf-8')
-                self.transport("term_stdin", {'data': data, 'id': self.term_id})
+                self.transport("term_stdin", {'data': base64.b64encode(data).decode('utf8'), 'id': self.term_id})
 
         if 'term_stdin' in self.ri['perms']:
             out('You have permission to write to this terminal. Remember: With great power comes great responsibility.')
@@ -685,7 +689,7 @@ class Flootty(object):
             out('You do not have permission to write to this terminal.')
 
         def stdout_write(buf):
-            write(stdout, buf)
+            write(stdout, base64.b64decode(buf))
 
         self.handle_stdio = stdout_write
         self._set_pty_size(self.ri['terms'][str(self.term_id)]['size'])
@@ -698,10 +702,10 @@ class Flootty(object):
 
         if self.master_fd:
             # reconnected. don't spawn a new shell
-            out('Reconnected to %s' % (self.room_url()))
+            out('Reconnected to %s' % (self.workspace_url()))
             return
         shell = os.environ['SHELL']
-        out('Successfully joined %s' % (self.room_url()))
+        out('Successfully joined %s' % (self.workspace_url()))
 
         self.child_pid, self.master_fd = pty.fork()
         if self.child_pid == pty.CHILD:
@@ -721,29 +725,14 @@ class Flootty(object):
             '''
             Called when there is data to be sent from the child process back to the user.
             '''
-
             try:
                 data = self.extra_data + os.read(fd, FD_READ_BYTES)
             except:
                 data = None
-
             if not data:
                 return die("Time to go!")
 
-            self.extra_data = b''
-
-            while True:
-                try:
-                    data.decode('utf-8')
-                except UnicodeDecodeError:
-                    self.extra_data = data[-1:] + self.extra_data
-                    data = data[:-1]
-                else:
-                    break
-                if len(self.extra_data) > 100:
-                    die('not a valid utf-8 string: %s' % self.extra_data)
-
-            self.transport("term_stdout", {'data': data.decode('utf-8'), 'id': self.term_id})
+            self.transport("term_stdout", {'data': base64.b64encode(data).decode('utf8'), 'id': self.term_id})
             write(pty.STDOUT_FILENO, data)
 
         self.add_fd(self.master_fd, reader=stdout_write, errer=slave_death, name='create_term_stdout_write')
@@ -779,7 +768,7 @@ class Flootty(object):
             color_reset = "%{%f%}"
 
         # Set prompt
-        cmd = 'PS1="%s%s::%s::%s%s $PS1"\n' % (color_start, self.owner, self.room, self.term_name, color_reset)
+        cmd = 'PS1="%s%s::%s::%s%s $PS1"\n' % (color_start, self.owner, self.workspace, self.term_name, color_reset)
         write(self.master_fd, cmd)
 
     def _signal_winch(self, signum, frame):
@@ -827,7 +816,6 @@ class Flootty(object):
         except Exception:
             pass
         print('ciao.')
-        sys.exit()
 
 if __name__ == '__main__':
     main()
